@@ -1,7 +1,7 @@
-/* 	$Id: ssl.c,v 1.2 1995/07/27 12:40:13 martin Exp $	 */
+/* 	$Id: ssl.c,v 1.4 1995/11/23 15:31:13 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: ssl.c,v 1.2 1995/07/27 12:40:13 martin Exp $";
+static char vcid[] = "$Id: ssl.c,v 1.4 1995/11/23 15:31:13 martin Exp martin $";
 #endif /* lint */
 
 #include "../../config.h"
@@ -26,7 +26,8 @@ static char vcid[] = "$Id: ssl.c,v 1.2 1995/07/27 12:40:13 martin Exp $";
  * 01-Jul-95 tjh    merged patches from Steven Schoch 
  * ................ <schoch@sheba.arc.nasa.gov> that add in the certsok
  * ................ option for using signed certificates rather than 
- * ................ explicit passwords for authentication (modified a little)
+ * ................ explicit passwords for authentication (modified a little
+ * ................ to add in an option that controls this feature)
  * 26-Apr-95 tjh    original coding
  *
  * tjh@mincom.oz.au
@@ -52,6 +53,7 @@ static char vcid[] = "$Id: ssl.c,v 1.2 1995/07/27 12:40:13 martin Exp $";
 #include "auth.h"
 #include "misc.h"
 
+#include "x509.h"
 #include "ssl.h"
 
 extern int auth_debug_mode;
@@ -81,6 +83,36 @@ static unsigned char str_data[1024] = { IAC, SB, TELOPT_AUTHENTICATION, 0,
 #define AUTH_SSL_ACCEPT    2
 #define AUTH_SSL_REJECT    3
 
+
+/* this is called by both the ssl.c auth connect and the mainline
+ * telnet connect if we are talking straight ssl with no telnet
+ * protocol --tjh
+ */
+        void
+display_connect_details(ssl_con,verbose)
+                        SSL *ssl_con;
+                        int verbose;
+{
+    X509 *peer;
+
+    fprintf(stderr,"[SSL Connected - Cipher %s]\n",SSL_get_cipher(ssl_con));
+    peer=SSL_get_peer_certificate(ssl_con);
+    if (peer != NULL) {
+	char *str;
+
+	str=X509_NAME_oneline(X509_get_subject_name(peer));
+	fprintf(stderr,"[SSL subject=%s]\n",str);
+	free(str);
+	str=X509_NAME_oneline(X509_get_issuer_name(peer));
+	fprintf(stderr,"[SSL issuer=%s]\n",str);
+	free(str);
+	X509_free(peer);
+			
+    }
+    fflush(stderr);
+}
+
+
 	void
 fprintd(fp, data, cnt)
 	FILE *fp;
@@ -235,27 +267,30 @@ int cnt;
 	case AUTH_SSL_ACCEPT:
 		if (auth_debug_mode)
 			fprintf(stderr,"SSL ACCEPT\r\n");
-		fprintf(stderr,"[ SSL starting ]\r\n");
+		fprintf(stderr,"[SSL starting]\r\n");
 
 		auth_finished(ap, AUTH_VALID);
 
 		if (ssl_dummy_flag) {
-		    fprintf(stderr,"[ SSL Dummy Connected ]\r\n");
+		    fprintf(stderr,"[SSL Dummy Connected]\r\n");
 		    fflush(stderr);
 		    return;
 		}
 
 		/* right ... now we drop into the SSL library */
 		if (!ssl_only_flag) {
-		    if (!SSL_connect(ssl_con)) {
+		    if (SSL_connect(ssl_con) <= 0) {
 			perror("telnet: Unable to ssl_connect to remote host");
 
-			/* don't know what I should be doing here ... */
+			SSL_load_error_strings();
+			ERR_print_errors(stderr);
+
+			/* don't know what I "should" be doing here ... */
+
 			auth_finished(0,AUTH_REJECT);
 			return;
 		    } else {
-			fprintf(stderr,"[ SSL Connected - Cipher %s]\r\n",SSL_get_cipher(ssl_con));
-			fflush(stderr);
+			display_connect_details(ssl_con,1);
 			ssl_active_flag=1;
 		    }
 		}
@@ -270,7 +305,7 @@ int cnt;
 	case AUTH_SSL_REJECT:
 		if (auth_debug_mode)
 			fprintf(stderr,"SSL REJECT\r\n");
-		fprintf(stderr,"[ SSL negotiation failed ]\r\n");
+		fprintf(stderr,"[SSL negotiation failed]\r\n");
 		fprintf(stderr,"Trying plaintext login:\r\n");
 		auth_finished(0,AUTH_REJECT);
 		break;
@@ -301,7 +336,7 @@ int level;
 	 */
 	if (ssl_certsok_flag) {
 	    user_fp = fopen("/etc/ssl.users", "r");
-	    if ( (!auth_ssl_name || !user_fp) && ssl_certsok_flag ) {
+	    if (!auth_ssl_name || !user_fp) {
 	        /* If we haven't received a certificate, then don't 
 		 * return AUTH_VALID. 
 		 */
@@ -320,10 +355,10 @@ int level;
 		 * to be able to add a little novel in files and
 		 * also disable easily --tjh
 		 */
-		if (cp[0]=='#')
+		if (buf[0]=='#')
 		    continue;
 
-		if (cp = strchr(buf, '\n'))
+		if ((cp = strchr(buf, '\n')))
 		    *cp = '\0';
 		cp = strchr(buf, ':');
 		if (!cp)
@@ -428,11 +463,12 @@ int depth, error;
 		free(auth_ssl_name);
 		auth_ssl_name = 0;
 	}
-	return ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? 0 : 1;
+	/* return ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? 0 : 1; */
+	return 0;
     }
     if (depth == 0)
 	auth_ssl_name =
-	    (char *)X509_oneline_X509_NAME(X509_get_subject_name(xs));
+	    (char *)X509_NAME_oneline(X509_get_subject_name(xs));
     return ok;
 }
 
