@@ -35,10 +35,10 @@
 static char sccsid[] = "@(#)commands.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: commands.c,v 1.3 1995/08/25 13:19:25 martin Exp $	 */
+/* 	$Id: commands.c,v 1.5 1995/11/23 15:42:30 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: commands.c,v 1.3 1995/08/25 13:19:25 martin Exp $";
+static char vcid[] = "$Id: commands.c,v 1.5 1995/11/23 15:42:30 martin Exp martin $";
 #endif /* lint */
 
 #include "../config.h"
@@ -53,7 +53,16 @@ static char vcid[] = "$Id: commands.c,v 1.3 1995/08/25 13:19:25 martin Exp $";
 #else
 #include <sys/types.h>
 #endif	/* defined(unix) */
-#include <sys/socket.h>
+#include <stdlib.h>
+#include <unistd.h>
+#if HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
+#if defined(__ultrix)
+# include <cxx/sys/socket.h>
+#else
+# include <sys/socket.h>
+#endif
 #include <netinet/in.h>
 #ifdef	CRAY
 #include <fcntl.h>
@@ -76,6 +85,8 @@ static char vcid[] = "$Id: commands.c,v 1.3 1995/08/25 13:19:25 martin Exp $";
 #include "defines.h"
 #include "types.h"
 
+#include "../lib/libtelnet/misc-proto.h"
+
 #if !defined(CRAY) && !defined(sysV88)
 #include <netinet/in_systm.h>
 # if (defined(vax) || defined(tahoe) || defined(hp300)) && !defined(ultrix)
@@ -86,8 +97,7 @@ static char vcid[] = "$Id: commands.c,v 1.3 1995/08/25 13:19:25 martin Exp $";
 
 #ifdef USE_SSL
 
-#include "X509.h"
-
+SSL_CTX *ssl_ctx;
 SSL *ssl_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
@@ -97,6 +107,9 @@ int ssl_secure_flag=0;
 int ssl_certsok_flag=0;
 char *cert_file=NULL;
 char *key_file=NULL;
+char *cipher_list=NULL;
+
+extern int display_connect_details();
 
 #endif /* USE_SSL */
 
@@ -117,7 +130,7 @@ extern int isprefix();
 extern char **genget();
 extern int Ambiguous();
 
-static call();
+static int call();
 
 typedef struct {
 	char	*name;		/* command name */
@@ -145,7 +158,7 @@ makeargv()
 	margc++;
 	cp++;
     }
-    while (c = *cp) {
+    while ((c = *cp)) {
 	register int inquote = 0;
 	while (isspace(c))
 	    c = *++cp;
@@ -188,7 +201,7 @@ makeargv()
  * Todo:  1.  Could take random integers (12, 0x12, 012, 0b1).
  */
 
-	static
+	static int
 special(s)
 	register char *s;
 {
@@ -317,7 +330,6 @@ sendcmd(argc, argv)
 {
     int count;		/* how many bytes we are going to need to send */
     int i;
-    int question = 0;	/* was at least one argument a question */
     struct sendlist *s;	/* pointer to current command */
     int success = 0;
     int needconnect = 0;
@@ -401,32 +413,6 @@ send_esc()
     return 1;
 }
 
-    static int
-send_docmd(name)
-    char *name;
-{
-    return(send_tncmd(send_do, "do", name));
-}
-
-    static int
-send_dontcmd(name)
-    char *name;
-{
-    return(send_tncmd(send_dont, "dont", name));
-}
-    static int
-send_willcmd(name)
-    char *name;
-{
-    return(send_tncmd(send_will, "will", name));
-}
-    static int
-send_wontcmd(name)
-    char *name;
-{
-    return(send_tncmd(send_wont, "wont", name));
-}
-
     int
 send_tncmd(func, cmd, name)
     void	(*func)();
@@ -490,6 +476,32 @@ send_tncmd(func, cmd, name)
     return 1;
 }
 
+    static int
+send_docmd(name)
+    char *name;
+{
+    return(send_tncmd(send_do, "do", name));
+}
+
+    static int
+send_dontcmd(name)
+    char *name;
+{
+    return(send_tncmd(send_dont, "dont", name));
+}
+    static int
+send_willcmd(name)
+    char *name;
+{
+    return(send_tncmd(send_will, "will", name));
+}
+    static int
+send_wontcmd(name)
+    char *name;
+{
+    return(send_tncmd(send_wont, "wont", name));
+}
+
     static int
 send_help()
 {
@@ -1113,6 +1125,8 @@ dokludgemode()
     send_wont(TELOPT_LINEMODE, 1);
     send_dont(TELOPT_SGA, 1);
     send_dont(TELOPT_ECHO, 1);
+
+    return 0;
 }
 #endif
 
@@ -1164,12 +1178,14 @@ dolmmode(bit, on)
 
     int
 setmode(bit)
+    int bit;
 {
     return dolmmode(bit, 1);
 }
 
     int
 clearmode(bit)
+    int bit;
 {
     return dolmmode(bit, 0);
 }
@@ -1459,7 +1475,7 @@ extern int shell();
 #endif	/* !defined(TN3270) */
 
     /*VARARGS*/
-    static
+    static int
 bye(argc, argv)
     int  argc;		/* Number of arguments */
     char *argv[];	/* arguments */
@@ -1489,11 +1505,13 @@ bye(argc, argv)
 }
 
 /*VARARGS*/
+        int
 quit()
 {
 	(void) call(bye, "bye", "fromquit", 0);
 	Exit(0);
 	/*NOTREACHED*/
+	return 0;
 }
 
 /*VARARGS*/
@@ -1554,7 +1572,7 @@ getslc(name)
 		genget(name, (char **) SlcList, sizeof(struct slclist));
 }
 
-    static
+    static int
 slccmd(argc, argv)
     int  argc;
     char *argv[];
@@ -1651,6 +1669,7 @@ getenvcmd(name)
 		genget(name, (char **) EnvList, sizeof(struct envlist));
 }
 
+    int
 env_cmd(argc, argv)
     int  argc;
     char *argv[];
@@ -1717,7 +1736,7 @@ env_init()
 	extern char *index();
 
 	for (epp = environ; *epp; epp++) {
-		if (cp = index(*epp, '=')) {
+		if ((cp = index(*epp, '='))) {
 			*cp = '\0';
 			ep = env_define((unsigned char *)*epp,
 					(unsigned char *)cp+1);
@@ -1762,7 +1781,7 @@ env_define(var, value)
 {
 	register struct env_lst *ep;
 
-	if (ep = env_find(var)) {
+	if ((ep = env_find(var))) {
 		if (ep->var)
 			free(ep->var);
 		if (ep->value)
@@ -1788,7 +1807,7 @@ env_undefine(var)
 {
 	register struct env_lst *ep;
 
-	if (ep = env_find(var)) {
+	if ((ep = env_find(var))) {
 		ep->prev->next = ep->next;
 		if (ep->next)
 			ep->next->prev = ep->prev;
@@ -1806,7 +1825,7 @@ env_export(var)
 {
 	register struct env_lst *ep;
 
-	if (ep = env_find(var))
+	if ((ep = env_find(var)))
 		ep->export = 1;
 }
 
@@ -1816,7 +1835,7 @@ env_unexport(var)
 {
 	register struct env_lst *ep;
 
-	if (ep = env_find(var))
+	if ((ep = env_find(var)))
 		ep->export = 0;
 }
 
@@ -1860,16 +1879,16 @@ env_list()
 
 	unsigned char *
 env_default(init, welldefined)
-	int init;
+	int init; int welldefined;
 {
 	static struct env_lst *nep = NULL;
 
 	if (init) {
 		nep = &envlisthead;
-		return;
+		return(NULL);
 	}
 	if (nep) {
-		while (nep = nep->next) {
+		while ((nep = nep->next)) {
 			if (nep->export && (nep->welldefined == welldefined))
 				return(nep->var);
 		}
@@ -1883,7 +1902,7 @@ env_getvalue(var)
 {
 	register struct env_lst *ep;
 
-	if (ep = env_find(var))
+	if ((ep = env_find(var)) != 0)
 		return(ep->value);
 	return(NULL);
 }
@@ -1973,6 +1992,7 @@ auth_help()
     return 0;
 }
 
+    int
 auth_cmd(argc, argv)
     int  argc;
     char *argv[];
@@ -2166,7 +2186,7 @@ filestuff(fd)
  * Print status about the connection.
  */
     /*ARGSUSED*/
-    static
+    static int
 status(argc, argv)
     int	 argc;
     char *argv[];
@@ -2245,7 +2265,9 @@ ayt_status()
 }
 #endif
 
-unsigned long inet_addr();
+unsigned long inet_addr P((const char *));
+void herror P((const char *));
+void cmdrc P((char *,char *));
 
     int
 tn(argc, argv)
@@ -2475,25 +2497,37 @@ tn(argc, argv)
 	    SSL_LOG=SSL_ERR=NULL;
 	}
 
-	ssl_con=(SSL *)SSL_new();
+	/* init things so we will get meaningful error messages
+	 * rather than numbers 
+	 */
+	SSL_load_error_strings();
+
+	ssl_ctx=(SSL *)SSL_CTX_new();
+	ssl_con=(SSL *)SSL_new(ssl_ctx);
 
         SSL_set_fd(ssl_con,net);
         SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-	SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
+	if (cipher_list==NULL)
+	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
+	else
+	    SSL_set_pref_cipher(ssl_con,cipher_list);
 
 	/* Add in any certificates if you want to here ... */
 	if (cert_file) {
 	    if (!SSL_use_certificate_file(ssl_con, cert_file,
-		X509_FILETYPE_PEM))
-		fprintf(stderr,"%s: %s\n",cert_file,
-		    SSL_error_string(SSL_errno));
-	    else {
+		X509_FILETYPE_PEM)) {
+		fprintf(stderr,"%s: ",cert_file);
+		ERR_print_errors(stderr);
+		exit(1);
+	    } else {
 		if (!key_file)
 		    key_file = cert_file;
 		if (!SSL_use_RSAPrivateKey_file(ssl_con, key_file,
-		    X509_FILETYPE_PEM))
-		    fprintf(stderr,"%s: %s\n", key_file,
-			SSL_error_string(SSL_errno));
+		    X509_FILETYPE_PEM)) {
+		    fprintf(stderr,"%s: ", key_file);
+		    ERR_print_errors(stderr);
+		    exit(1);
+		}
 	    }
 	}
 
@@ -2501,15 +2535,13 @@ tn(argc, argv)
 	    if (!SSL_connect(ssl_con)) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"telnet: SSL_connect error %s %s\n",
-		            SSL_error_func_string(SSL_errno),
-			    SSL_error_string(SSL_errno));
+		ERR_print_errors(stderr);
+		perror("SSL_connect");
+		fflush(stderr);
 
-		perror(errbuf);
 		return 0;
 	    } else {
-		fprintf(stderr,"[SSL Cipher %s]\n",SSL_get_cipher(ssl_con));
-		fflush(stderr);
+		display_connect_details(ssl_con,1);
 		ssl_active_flag=1;
 	    }
 	}
@@ -2525,8 +2557,8 @@ tn(argc, argv)
 
 	user = getenv("USER");
 	if (user == NULL ||
-	    (pw = getpwnam(user)) && pw->pw_uid != getuid()) {
-		if (pw = getpwuid(getuid()))
+	    ((pw = getpwnam(user)) && pw->pw_uid != getuid())) {
+		if ((pw = getpwuid(getuid())))
 			user = pw->pw_name;
 		else
 			user = NULL;
@@ -2542,6 +2574,7 @@ tn(argc, argv)
     (void) NetClose(net);
     ExitString("Connection closed by foreign host.\n",1);
     /*NOTREACHED*/
+    return 0;
 }
 
 #define HELPINDENT (sizeof ("connect"))
@@ -2609,7 +2642,7 @@ static Command cmdtab[] = {
 #endif
 	{ "environ",	envhelp,	env_cmd,	0 },
 	{ "?",		helphelp,	help,		0 },
-	0
+	{ 0 }
 };
 
 static char	crmodhelp[] =	"deprecated command -- use 'toggle crmod' instead";
@@ -2619,7 +2652,7 @@ static Command cmdtab2[] = {
 	{ "help",	0,		help,		0 },
 	{ "escape",	escapehelp,	setescape,	0 },
 	{ "crmod",	crmodhelp,	togcrmod,	0 },
-	0
+	{ 0 }
 };
 
 
@@ -2628,7 +2661,7 @@ static Command cmdtab2[] = {
  */
 
     /*VARARGS1*/
-    static
+    static int
 call(va_alist)
     va_dcl
 {
@@ -2654,7 +2687,7 @@ getcmd(name)
 {
     Command *cm;
 
-    if (cm = (Command *) genget(name, (char **) cmdtab, sizeof(Command)))
+    if ((cm = (Command *) genget(name, (char **) cmdtab, sizeof(Command))))
 	return cm;
     return (Command *) genget(name, (char **) cmdtab2, sizeof(Command));
 }
@@ -2743,7 +2776,7 @@ command(top, tbuf, cnt)
 /*
  * Help command.
  */
-	static
+	static int
 help(argc, argv)
 	int argc;
 	char *argv[];
@@ -2776,6 +2809,7 @@ help(argc, argv)
 static char *rcname = 0;
 static char rcbuf[128];
 
+        void
 cmdrc(m1, m2)
 	char *m1, *m2;
 {
@@ -2969,7 +3003,7 @@ sourceroute(arg, cpp, lenp)
 	for (c = 0;;) {
 		if (c == ':')
 			cp2 = 0;
-		else for (cp2 = cp; c = *cp2; cp2++) {
+		else for ((cp2 = cp); (c = *cp2); cp2++) {
 			if (c == ',') {
 				*cp2++ = '\0';
 				if (*cp2 == '@')
@@ -2987,7 +3021,7 @@ sourceroute(arg, cpp, lenp)
 
 		if ((tmp = inet_addr(cp)) != -1) {
 			sin_addr.s_addr = tmp;
-		} else if (host = gethostbyname(cp)) {
+		} else if ((host = gethostbyname(cp))) {
 #if	defined(h_addr)
 			memcpy((caddr_t)&sin_addr,
 				host->h_addr_list[0], host->h_length);
