@@ -33,7 +33,7 @@
  *	@(#)externs.h	8.2 (Berkeley) 12/15/93
  */
 
-/* 	$Id: externs.h,v 1.3 1995/08/25 13:20:56 martin Exp $	 */
+/* 	$Id: externs.h,v 1.5 1995/11/23 15:42:44 martin Exp $	 */
 
 #ifndef	BSD
 # define BSD 43
@@ -66,7 +66,7 @@
 # include <errno.h>
 #endif /* CRAY */
 #ifdef	USE_TERMIO
-# ifndef	VINTR
+# if !defined(VINTR) || defined(LINUX)
 #  ifdef SYSV_TERMIO
 #   include <termio.h>
 #  else
@@ -247,6 +247,8 @@ extern FILE
     *NetTrace;		/* Where debugging output goes */
 extern unsigned char
     NetTraceFile[];	/* Name of file where debugging output goes */
+extern int
+    SetSockOpt P((int, int, int, int)); /* Compensate for differences in 4.2 and 4.3 systems.*/
 extern void
     SetNetTrace P((char *));	/* Function to change where debugging goes */
 
@@ -259,7 +261,7 @@ extern void
     Dump P((int, unsigned char *, int)),
     init_3270 P((void)),
     printoption P((char *, int, int)),
-    printsub P((int, unsigned char *, int)),
+    printsub P((char, unsigned char *, int)),
     sendnaws P((void)),
     setconnmode P((int)),
     setcommandmode P((void)),
@@ -267,6 +269,7 @@ extern void
     sys_telnet_init P((void)),
     telnet P((char *)),
     tel_enter_binary P((int)),
+    tel_leave_binary P((int)),
     TerminalFlushOutput P((void)),
     TerminalNewMode P((int)),
     TerminalRestoreState P((void)),
@@ -274,7 +277,38 @@ extern void
     tninit P((void)),
     upcase P((char *)),
     willoption P((int)),
-    wontoption P((int));
+    wontoption P((int)),
+    optionstatus P((void)),
+    ExitString P((char *, int)),
+    Exit P((int)),
+    init_terminal P((void)),
+    init_network P((void)),
+    init_telnet P((void)),
+    init_sys P((void)),
+    set_escape_char P((char *)),
+    sendabort P((void)),
+    sendeof P((void)),
+    sendsusp P((void)),
+    env_init P((void)),
+    TerminalSpeeds P((long *, long *)),
+    TerminalDefaultChars P((void));
+
+extern int
+    TerminalWindowSize P((long *, long *)),
+    NetClose P((int)),
+    opt_welldefined P((char *)),
+    getconnmode P((void)),
+    tn P((int, char *[])),
+    ring_at_mark P((Ring *)),
+    rlogin_susp P((void)),
+    ttyflush P((int)),
+    TerminalAutoFlush P((void)),
+    TerminalWrite P((char *, int)),
+    quit P((void)),
+    TerminalSpecialChars P((int)),
+    process_rings P((int, int, int, int, int, int)),
+    telrcv P((void)),
+    stilloob P((void));
 
 extern void
     send_do P((int, int)),
@@ -497,6 +531,7 @@ extern void
 #endif	/* defined(TN3270) */
 
 #ifdef USE_SSL
+#include "x509.h"
 #include "ssl.h"
 extern SSL *ssl_con;
 extern int ssl_debug_flag;
@@ -506,6 +541,7 @@ extern int ssl_verify_flag;
 extern int ssl_secure_flag;
 extern char *cert_file; 
 extern char *key_file;
+extern char *cipher_list;
 #endif /* USE_SSL */
 
 #ifdef USE_SOCKS
