@@ -41,20 +41,24 @@ static char copyright[] =
 static char sccsid[] = "@(#)main.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: main.c,v 1.2 1995/07/27 12:44:19 martin Exp $	 */
+/* 	$Id: main.c,v 1.4 1995/11/23 15:42:55 martin Exp $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: main.c,v 1.2 1995/07/27 12:44:19 martin Exp $";
+static char vcid[] = "$Id: main.c,v 1.4 1995/11/23 15:42:55 martin Exp $";
 #endif /* lint */
 
 #include "../config.h"
 
+#include <stdlib.h>
 #include <sys/types.h>
 
 #include "ring.h"
 #include "externs.h"
 #include "defines.h"
 
+#include "../lib/libtelnet/auth.h"
+#include "../lib/libtelnet/auth-proto.h"
+
 /* These values need to be the same as defined in libtelnet/kerberos5.c */
 /* Either define them in both places, or put in some common header file. */
 #define OPTS_FORWARD_CREDS           0x00000002
@@ -65,7 +69,7 @@ static char vcid[] = "$Id: main.c,v 1.2 1995/07/27 12:44:19 martin Exp $";
 #endif
 
 #ifdef USE_SOCKS
-int socks_flag=0;
+int socks_flag=1;
 #endif /* USE_SOCKS */
 
 /*
@@ -90,7 +94,7 @@ tninit()
 	void
 usage()
 {
-	fprintf(stderr, "Usage: %s %s%s%s%s\n",
+	fprintf(stderr, "Usage: %s %s%s%s%s%s\n",
 	    prompt,
 #ifdef	AUTHENTICATION
 	    "[-8] [-E] [-K] [-L] [-S tos] [-X atype] [-a] [-c] [-d] [-e char]",
@@ -127,7 +131,7 @@ usage()
  * main.  Parse arguments, invoke the protocol or command parser.
  */
 
-
+        int
 main(argc, argv)
 	int argc;
 	char *argv[];
@@ -147,7 +151,7 @@ main(argc, argv)
 
 	TerminalSaveState();
 
-	if (prompt = strrchr(argv[0], '/'))
+	if ((prompt = strrchr(argv[0], '/')))
 		++prompt;
 	else
 		prompt = argv[0];
@@ -185,10 +189,19 @@ main(argc, argv)
 		        if (strncmp(optarg, "key=", sizeof("key")) == 0 ) {
 			    key_file= optarg + sizeof("key");
 			}
+		        if (strncmp(optarg,"cipher=",sizeof("cipher"))==0) {
+			    cipher_list=optarg+sizeof("cipher");
+			}
 #ifdef USE_SOCKS
 		        if (strcmp(optarg, "socks") == 0 ) {
 			    socks_flag=1;
 			}
+		        if (strcmp(optarg, "!socks") == 0 ) {
+			    socks_flag=0;
+			}
+#else /* !USE_SOCKS */
+		        fprintf(stderr,"No SOCKS support\n");
+ 			fflush(stderr);
 #endif /* USE_SOCKS */
 			break;
 #endif /* USE_SSL */
@@ -199,8 +212,14 @@ main(argc, argv)
 		        if (strcmp(optarg, "socks") == 0 ) {
 			    socks_flag=1;
 			}
+		        if (strcmp(optarg, "!socks") == 0 ) {
+			    socks_flag=0;
+			}
 			break;
 #endif /* !USE_SSL */
+#else /* !USE_SOCKS */
+		        fprintf(stderr,"No SOCKS support\n");
+ 			fflush(stderr);
 #endif /* USE_SOCKS */
 
 
