@@ -35,10 +35,10 @@
 static char sccsid[] = "@(#)telnet.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: telnet.c,v 1.2 1995/07/27 12:48:00 martin Exp $	 */
+/* 	$Id: telnet.c,v 1.3 1995/11/15 08:40:03 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: telnet.c,v 1.2 1995/07/27 12:48:00 martin Exp $";
+static char vcid[] = "$Id: telnet.c,v 1.3 1995/11/15 08:40:03 martin Exp martin $";
 #endif /* lint */
 
 #include "../config.h"
@@ -47,15 +47,22 @@ static char vcid[] = "$Id: telnet.c,v 1.2 1995/07/27 12:48:00 martin Exp $";
 
 #if	defined(unix)
 #include <signal.h>
+#endif	/* defined(unix) */
+
 /* By the way, we need to include curses.h before telnet.h since,
  * among other things, telnet.h #defines 'DO', which is a variable
  * declared in curses.h.
  */
-#endif	/* defined(unix) */
+
+#ifndef TERMCAP
+#include <curses.h>
+#endif
 
 #include <arpa/telnet.h>
 
 #include <ctype.h>
+#include <stdlib.h>
+#include <unistd.h>
 
 #include "ring.h"
 
@@ -64,6 +71,10 @@ static char vcid[] = "$Id: telnet.c,v 1.2 1995/07/27 12:48:00 martin Exp $";
 #include "types.h"
 #include "general.h"
 
+#include "../lib/libtelnet/misc-proto.h"
+#include "../lib/libtelnet/auth.h"
+#include "../lib/libtelnet/auth-proto.h"
+
 
 #define	strip(x)	((x)&0x7f)
 
@@ -638,6 +649,11 @@ dontoption(option)
 	set_my_state_wont(option);
 }
 
+	int
+is_unique(register char *name,
+	  register char **as,
+	  register char **ae);
+
 /*
  * Given a buffer returned by tgetent(), this routine will turn
  * the pipe seperated list of names in the buffer into an array
@@ -1245,7 +1261,7 @@ slc_init()
 
 #define	initfunc(func, flags) { \
 					spcp = &spc_data[func]; \
-					if (spcp->valp = tcval(func)) { \
+					if ((spcp->valp = tcval(func))) { \
 					    spcp->val = *spcp->valp; \
 					    spcp->mylevel = SLC_VARIABLE|flags; \
 					} else { \
@@ -1653,12 +1669,12 @@ env_opt_add(ep)
 	if (ep == NULL || *ep == '\0') {
 		/* Send user defined variables first. */
 		env_default(1, 0);
-		while (ep = env_default(0, 0))
+		while ((ep = env_default(0, 0)))
 			env_opt_add(ep);
 
 		/* Now add the list of well know variables.  */
 		env_default(1, 1);
-		while (ep = env_default(0, 1))
+		while ((ep = env_default(0, 1)))
 			env_opt_add(ep);
 		return;
 	}
@@ -1688,7 +1704,7 @@ env_opt_add(ep)
 	else
 		*opt_replyp++ = ENV_USERVAR;
 	for (;;) {
-		while (c = *ep++) {
+		while ((c = *ep++)) {
 			switch(c&0xff) {
 			case IAC:
 				*opt_replyp++ = IAC;
@@ -1702,7 +1718,7 @@ env_opt_add(ep)
 			}
 			*opt_replyp++ = c;
 		}
-		if (ep = vp) {
+		if ((ep = vp)) {
 #ifdef	OLD_ENVIRON
 			if (telopt_environ == TELOPT_OLD_ENVIRON)
 				*opt_replyp++ = old_env_value;
