@@ -35,14 +35,17 @@
 static char sccsid[] = "@(#)utilities.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: utilities.c,v 1.3 1995/08/25 13:24:22 martin Exp $	 */
+/* 	$Id: utilities.c,v 1.4 1995/11/15 09:14:42 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: utilities.c,v 1.3 1995/08/25 13:24:22 martin Exp $";
+static char vcid[] = "$Id: utilities.c,v 1.4 1995/11/15 09:14:42 martin Exp martin $";
 #endif /* lint */
 
 #include "../config.h"
 
+#include "../lib/libtelnet/auth.h"
+#include "../lib/libtelnet/auth-proto.h"
+
 #define	TELOPTS
 #define	TELCMDS
 #define	SLC_NAMES
@@ -61,6 +64,11 @@ static char vcid[] = "$Id: utilities.c,v 1.3 1995/08/25 13:24:22 martin Exp $";
 #if defined(HAVE_SYS_SELECT_H) && !defined(SCO)
 # include <sys/select.h>
 #endif
+#if defined(__ultrix)
+# include <cxx/sys/socket.h>
+#else
+# include <sys/socket.h>
+#endif
 
 #include <ctype.h>
 
@@ -154,7 +162,7 @@ Dump(direction, buffer, length)
 #   define min(x,y)	((x<y)? x:y)
     unsigned char *pThis;
     int offset;
-    extern pettydump;
+    /*    extern pettydump;*/
 
     offset = 0;
 
