@@ -33,7 +33,7 @@
  *	@(#)ext.h	8.2 (Berkeley) 12/15/93
  */
 
-/* 	$Id: ext.h,v 1.2 1995/07/27 12:52:47 martin Exp $	 */
+/* 	$Id: ext.h,v 1.4 1995/11/23 15:43:21 martin Exp $	 */
 
 /*
  * Telnet server variable declarations
@@ -123,11 +123,10 @@ extern void
 	localstat P((void)),
 	flowstat P((void)),
 	netclear P((void)),
-	netflush P((void)),
 #ifdef DIAGNOSTICS
 	printoption P((char *, int)),
 	printdata P((char *, char *, int)),
-	printsub P((int, unsigned char *, int)),
+	printsub P((char, unsigned char *, int)),
 #endif
 	ptyflush P((void)),
 	putchr P((int)),
@@ -144,10 +143,10 @@ extern void
 	set_termbuf P((void)),
 	start_login P((char *, int, char *)),
 	start_slc P((int)),
-#if	defined(AUTHENTICATION)
-	start_slave P((char *)),
+#ifdef  convex
+	startslave P((char *)),
 #else
-	start_slave P((char *, int, char *)),
+	startslave P((char *, int, char *)),
 #endif
 	suboption P((void)),
 	telrcv P((void)),
@@ -158,6 +157,7 @@ extern void
 extern int
 	end_slc P((unsigned char **)),
 	getnpty P((void)),
+	netflush P((void)),
 #ifndef convex
 	getpty P((int *)),
 #endif
@@ -256,8 +256,11 @@ extern int	needtermstat;
 #endif /* USE_SSL */
 
 #ifdef USE_SSL
+
+#include "x509.h"
 #include "ssl.h"
-#include "X509.h"
+
+extern SSL_CTX *ssl_ctx;
 extern SSL *ssl_con;
 extern int ssl_only_flag;
 extern int ssl_debug_flag;
