@@ -35,10 +35,10 @@
 static char sccsid[] = "@(#)sys_term.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: sys_term.c,v 1.3 1995/08/25 13:28:23 martin Exp $	 */
+/* 	$Id: sys_term.c,v 1.5 1995/11/23 15:43:35 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: sys_term.c,v 1.3 1995/08/25 13:28:23 martin Exp $";
+static char vcid[] = "$Id: sys_term.c,v 1.5 1995/11/23 15:43:35 martin Exp martin $";
 #endif /* lint */
 
 #include "telnetd.h"
@@ -47,6 +47,7 @@ static char vcid[] = "$Id: sys_term.c,v 1.3 1995/08/25 13:28:23 martin Exp $";
 #if	defined(AUTHENTICATION)
 #include <libtelnet/auth.h>
 #endif
+#include <syslog.h>
 
 #if defined(CRAY) || defined(__hpux) || defined(SCO) || defined(_AIX)
 # define PARENT_DOES_UTMP
@@ -98,6 +99,7 @@ extern struct sysv sysv;
 #endif	/* NEWINIT */
 
 #ifdef	STREAMSPTY
+#include <sys/stream.h>
 # ifdef HAVE_SAC_H
 #  include <sac.h>
 # endif
@@ -105,7 +107,9 @@ extern struct sysv sysv;
 # ifdef HAVE_SYS_STRTTY_H
 #  include <sys/strtty.h>
 # endif
-# include <sys/tty.h>
+# ifdef HAVE_SYS_TTY_H
+#  include <sys/tty.h>
+# endif
 # ifdef HAVE_SYS_PTYVAR_H
 #  include <sys/ptyvar.h>
 # endif
@@ -114,15 +118,22 @@ extern struct sysv sysv;
 #define SCPYN(a, b)	(void) strncpy(a, b, sizeof(a))
 #define SCMPN(a, b)	strncmp(a, b, sizeof(a))
 
-#ifdef	STREAMS
-#include <sys/stream.h>
+/* if we do the following then we can build for IRIX 
+ * without STREAMSPTY support --tjh
+ */
+#if defined(sgi)
+#include <sac.h>            /* SC_WILDC */
+#include <sys/strtty.h>     /* EXTPROC */
 #endif
+
 #ifdef __hpux
 #include <sys/resource.h>
 #include <sys/proc.h>
-#endif
-#ifdef HAVE_SYS_TTY_H
-# include <sys/tty.h>
+#include <sys/tty.h>
+/* note: <sys/tty.h> under HP-UX 9.04 wants <sys/timeout.h> which
+ *       doesn't exist on my test machine so I've put a dummy one
+ *       in ../include/sys --tjh
+ */
 #endif
 #ifdef	t_erase
 #undef	t_erase
@@ -150,6 +161,12 @@ extern struct sysv sysv;
 #if defined(linux)
 # define EXTPROC 0400
 #endif
+#if defined(__alpha)
+# define EXTPROC 0400
+#endif
+#if defined(__ultrix)
+# define TS_EXTPROC 0400
+#endif
 
 #ifdef __hpux
 #undef TIOCSCTTY
@@ -221,6 +238,7 @@ int ttyfd = -1;
 	void
 init_termbuf()
 {
+        int ret;
 #ifndef	USE_TERMIO
 	(void) ioctl(pty, TIOCGETP, (char *)&termbuf.sg);
 	(void) ioctl(pty, TIOCGETC, (char *)&termbuf.tc);
@@ -230,9 +248,9 @@ init_termbuf()
 # endif
 #else
 # ifdef  STREAMSPTY
-	(void) tcgetattr(ttyfd, &termbuf);
+	ret = tcgetattr(ttyfd, &termbuf);
 # else
-	(void) tcgetattr(pty, &termbuf);
+	ret = tcgetattr(pty, &termbuf);
 # endif
 #endif
 	termbuf2 = termbuf;
@@ -493,23 +511,129 @@ getnpty()
  *
  * Returns the file descriptor of the opened pty.
  */
-#ifndef	__GNUC__
-char *line = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
-#else
 static char Xline[] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 char *line = Xline;
+
+/* the following shouldn't be needed and will probably be
+ * deleted shortly as the GCC + SGI way works for everything
+ * so why make a difference now that I've sorted out the
+ * mess in ext.h with global.c --tjh
+ */
+#if 0
+static char Xline[] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
+char *line = Xline;
+#if defined(__GNUC__) || defined(__sgi) 
+#else
+char *line = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 #endif
+#endif
+
 #ifdef	CRAY
 char *myline = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
 #endif	/* CRAY */
 
-	int
+/*
+** allow for mobility of the pty master/slave directories
+*/
+#ifndef PTYDEV
+#ifdef __hpux
+#define	PTYDEV		"/dev/ptym/ptyxx"
+#else	/* !__hpux */
+#define	PTYDEV		"/dev/ptyxx"
+#endif	/* !__hpux */
+#endif	/* !PTYDEV */
+
+#ifndef TTYDEV
+#ifdef hpux
+#define TTYDEV		"/dev/pty/ttyxx"
+#else	/* !hpux */
+#define	TTYDEV		"/dev/ttyxx"
+#endif	/* !hpux */
+#endif	/* !TTYDEV */
+
+#ifndef PTYCHAR1
+#ifdef __hpux
+#define PTYCHAR1	"zyxwvutsrqp"
+#else	/* !__hpux */
+#define	PTYCHAR1	"pqrstuvwxyzPQRSTUVWXYZ"
+#endif	/* !__hpux */
+#endif	/* !PTYCHAR1 */
+
+#ifndef PTYCHAR2
+#ifdef __hpux
+#define	PTYCHAR2	"fedcba9876543210"
+#else	/* !__hpux */
+#define	PTYCHAR2	"0123456789abcdef"
+#endif	/* !__hpux */
+#endif	/* !PTYCHAR2 */
+
+/*
+ * Called from get_pty to iterate over likely pseudo terminals
+ * we might allocate.  Used on those systems that do not have
+ * a functional interface for allocating a pty.
+ * Returns 0 if found a pty, 1 if fails.
+ */
+int pty_search(pty)
+    int *pty;
+{
+    static int devindex, letter = 0;
+    struct stat stb;
+    char *ptydev;
+    char *ttydev;
+
+    ttydev = (char *) malloc (strlen (TTYDEV) + 1);
+    ptydev = (char *) malloc (strlen (PTYDEV) + 1);
+    strcpy (ttydev, TTYDEV);
+    strcpy (ptydev, PTYDEV);
+    
+    while (PTYCHAR1[letter]) {
+	ttydev [strlen(ttydev) - 2]  = ptydev [strlen(ptydev) - 2] =
+	    PTYCHAR1 [letter];
+
+	while (PTYCHAR2[devindex]) {
+	    ttydev [strlen(ttydev) - 1] = ptydev [strlen(ptydev) - 1] =
+		PTYCHAR2 [devindex];
+	    /* for next time around loop or next entry to this function */
+	    devindex++;
+	    if (stat(ptydev, &stb) < 0)
+	        break;
+	    if ((*pty = open (ptydev, O_RDWR)) >= 0) {
+#ifdef sun
+		/* Need to check the process group of the pty.
+		 * If it exists, then the slave pty is in use,
+		 * and we need to get another one.
+		 */
+		int pgrp_rtn;
+		if (ioctl(*pty, TIOCGPGRP, &pgrp_rtn) == 0 || errno != EIO) {
+		    close(*pty);
+		    continue;
+		}
+#endif /* sun */
+		strcpy(line,ttydev);
+		free(ttydev);
+		free(ptydev);
+		return 0;
+	    }
+	}
+	devindex = 0;
+	(void) letter++;
+    }
+    /*
+     * We were unable to allocate a pty master!  Return an error
+     * condition and let our caller terminate cleanly.
+     */
+    free(ttydev);
+    free(ptydev);
+    return 1;
+}
+
+int
 getpty(ptynum)
 int *ptynum;
 {
-	register int p;
+	int p;
 #ifdef	STREAMSPTY
-	int t;
+	/*	int t;*/
 	char *ptsname();
 
 	p = open("/dev/ptmx", 2);
@@ -528,6 +652,31 @@ int *ptynum;
 	int dummy;
 #endif
 
+#ifdef  _AIX
+	if ((p = open("/dev/ptc", O_RDWR)) < 0) {
+	  return -1;
+	}
+	strcpy(line, ttyname(p));
+	return p;
+#endif
+
+#if defined(sgi)
+	{
+	    char    *tty_name;
+
+	    tty_name = _getpty (&pty, O_RDWR, 0622, 0);
+	    if (tty_name == 0)
+		return -1;
+	    /* need this! --tjh */
+	    line = Xline;
+	    strcpy (line, tty_name);
+	    return 0;
+	}
+#endif
+	pty_search(&p);
+
+	return p;
+
 #ifndef	__hpux
 	(void) sprintf(line, "/dev/ptyXX");
 	p1 = &line[8];
@@ -1056,6 +1205,9 @@ extern void utmp_sig_notify P((int));
 # endif /* PARENT_DOES_UTMP */
 #endif
 
+	int
+cleanopen(char *line);
+
 /*
  * getptyslave()
  *
@@ -1063,7 +1215,7 @@ extern void utmp_sig_notify P((int));
  * that is necessary.  The return value is a file descriptor
  * for the slave side.
  */
-	int
+	void
 getptyslave()
 {
 	register int t = -1;
@@ -1085,6 +1237,7 @@ getptyslave()
 	 *	terminal speed
 	 * so that we can re-set them if we need to.
 	 */
+
 # ifdef	LINEMODE
 	waslm = tty_linemode();
 # endif
@@ -1109,7 +1262,6 @@ getptyslave()
 	 */
 	utmp_sig_wait();
 # endif
-
 	t = cleanopen(line);
 	if (t < 0)
 		fatalperror(net, line);
@@ -1122,15 +1274,18 @@ getptyslave()
 		fatal(net, "I_PUSH ptem");
 	if (ioctl(t, I_PUSH, "ldterm") < 0)
 		fatal(net, "I_PUSH ldterm");
+#ifndef sgi
 	if (ioctl(t, I_PUSH, "ttcompat") < 0)
 		fatal(net, "I_PUSH ttcompat");
 	if (ioctl(pty, I_PUSH, "pckt") < 0)
 		fatal(net, "I_PUSH pckt");
+#endif
 #endif
 
 	/*
 	 * set up the tty modes as we like them to be.
 	 */
+
 	init_termbuf();
 # ifdef	TIOCGWINSZ
 	if (def_row || def_col) {
@@ -1149,7 +1304,7 @@ getptyslave()
 # endif	/* USE_TERMIO */
 
 	/*
-	 * Settings for UNICOS (and HPUX)
+	 * Settings for UNICOS (and HPUX) (and ...?)
 	 */
 # if defined(CRAY) || defined(__hpux) || defined(SCO)
 	termbuf.c_oflag = OPOST|ONLCR|TAB3;
@@ -1264,6 +1419,7 @@ cleanopen(line)
 #ifdef HAVE_VHANGUP
 	vhangup();
 #endif
+
 	(void) signal(SIGHUP, SIG_DFL);
 	t = open(line, O_RDWR|O_NOCTTY);
 	if (t < 0)
@@ -1309,13 +1465,22 @@ cleanopen(line)
 }
 #endif	/* !defined(CRAY) || !defined(NEWINIT) */
 
-#if BSD <= 43
+#if BSD <= 43 
 
 	int
 login_tty(t)
 	int t;
 {
-	if (setsid() < 0) {
+	int res;
+
+	if ( (res=setsid()) < 0) {
+#if defined(_AIX)
+		if (debug) {
+			fprintf(stderr,"setsid: res=%d errno=%d\n",res,errno);
+			fflush(stderr);
+		}
+		goto ok;
+#endif
 #ifdef ultrix
 		/*
 		 * The setsid() may have failed because we
@@ -1326,6 +1491,8 @@ login_tty(t)
 #endif
 			fatalperror(net, "setsid()");
 	}
+ok:;
+
 # ifdef	TIOCSCTTY
 	if (ioctl(t, TIOCSCTTY, (char *)0) < 0)
 		fatalperror(net, "ioctl(sctty)");
@@ -1384,8 +1551,8 @@ startslave(host, autologin, autoname)
 	char *autoname;
 {
 	register int i;
-	long time();
-	char name[256];
+	/*long time();*/
+	/*	char name[256];*/
 #ifdef	NEWINIT
 	extern char *ptyip;
 	struct init_request request;
@@ -1520,7 +1687,7 @@ init_env()
 	char **envp;
 
 	envp = envinit;
-	if (*envp = getenv("TZ"))
+	if ((*envp = getenv("TZ")))
 		*envp++ -= 3;
 #if	defined(CRAY) || defined(__hpux)
 	else
@@ -1532,6 +1699,8 @@ init_env()
 
 #ifndef	NEWINIT
 
+void scrub_env P((void));
+
 /*
  * start_login(host)
  *
@@ -1545,7 +1714,7 @@ start_login(host, autologin, name)
 	int autologin;
 	char *name;
 {
-	register char *cp;
+  /*	register char *cp;*/
 	register char **argv;
 	char **addarg();
 	extern char *getenv();
@@ -1577,6 +1746,9 @@ start_login(host, autologin, name)
 		fatal(net, "makeutx failed");
 #endif
 
+	/* remove undesirables from the environment */
+        scrub_env();
+
 	/*
 	 * -h : pass on name of host.
 	 *		WARNING:  -h is accepted by login if and only if
@@ -1786,6 +1958,14 @@ addarg(argv, val)
 }
 #endif	/* NEWINIT */
 
+#ifdef __hpux
+        void
+rmut (char *line);
+#else
+        void
+rmut (void);
+#endif
+
 /*
  * cleanup()
  *
@@ -1862,7 +2042,11 @@ cleanup(sig)
 	t = cleantmp(&wtmp);
 	setutent();	/* just to make sure */
 #  endif /* CRAY */
-	rmut(line);
+#  ifdef __hpux
+        rmut(line);
+#  else
+        rmut();
+#  endif
 	close(pty);
 	(void) shutdown(net, 2);
 #  ifdef CRAY
@@ -1932,7 +2116,7 @@ utmp_sig_wait()
 }
 
 	void
-utmp_sig_notify(pid)
+utmp_sig_notify(int pid)
 {
 	kill(pid, SIGUSR1);
 }
@@ -2070,11 +2254,11 @@ cleantmpdir(jid, tpath, user)
 	void
 rmut()
 {
-	register f;
+  /*	register f;
 	int found = 0;
 	struct utmp *u, *utmp;
 	int nutmp;
-	struct stat statbf;
+	struct stat statbf;*/
 
 	struct utmpx *utxp, utmpx;
 
@@ -2157,6 +2341,7 @@ rmut()
 #endif	/* CRAY */
 
 #ifdef __hpux
+        void
 rmut (line)
 char *line;
 {
@@ -2194,3 +2379,27 @@ char *line;
 	(void) chown(line, 0, 0);
 }
 #endif
+
+
+/*
+ * scrub_env()
+ *
+ * Remove a few things from the environment that
+ * don't need to be there.
+ */
+      void
+scrub_env()
+{
+      register char **cpp, **cpp2;
+
+      for (cpp2 = cpp = environ; *cpp; cpp++) {
+              if (strncmp(*cpp, "LD_", 3) &&
+                  strncmp(*cpp, "_RLD_", 5) &&
+                  strncmp(*cpp, "ELF_LD_", 7) &&
+                  strncmp(*cpp, "LIBPATH=", 8) &&
+                  strncmp(*cpp, "IFS=", 4))
+                      *cpp2++ = *cpp;
+      }
+      *cpp2 = 0;
+}
+
