@@ -1,3 +1,9 @@
+/*
+#ifdef __sgi
+#define TJH_FORCED_FLUSH
+#endif
+*/
+
 /*
  * Copyright (c) 1989, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -41,15 +47,19 @@ static char copyright[] =
 static char sccsid[] = "@(#)telnetd.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: telnetd.c,v 1.2 1995/07/27 12:57:00 martin Exp $	 */
+/* 	$Id: telnetd.c,v 1.4 1995/11/23 15:43:46 martin Exp martin $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: telnetd.c,v 1.2 1995/07/27 12:57:00 martin Exp $";
+static char vcid[] = "$Id: telnetd.c,v 1.4 1995/11/23 15:43:46 martin Exp martin $";
 #endif /* lint */
 
 #include "telnetd.h"
 #include "pathnames.h"
 
+#include "../lib/libtelnet/misc-proto.h"
+
+#include <syslog.h>
+
 #if	defined(_SC_CRAY_SECURE_SYS) && !defined(SCM_SECURITY)
 /*
  * UNICOS 6.0/6.1 do not have SCM_SECURITY defined, so we can
@@ -108,7 +118,9 @@ int	registerd_host_only = 0;
 #  define TCSETSF TCSAFLUSH
 # endif
 /* make sure we don't get the bsd version */
-# include "/usr/include/sys/tty.h"
+# ifdef HAVE_SYS_TTY_H
+#  include "/usr/include/sys/tty.h"
+# endif
 # ifdef HAVE_SYS_PTYVAR_H
 #  include <sys/ptyvar.h>
 # endif
@@ -193,8 +205,7 @@ char valid_opts[] = {
 
 #ifdef USE_SSL
 
-#include "ssl_err.h"
-
+SSL_CTX *ssl_ctx;
 SSL *ssl_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
@@ -206,10 +217,16 @@ char ssl_file_path[1024];    /* don't look at that nasty value to the left */
 
 static char *my_ssl_key_file=NULL;
 static char *my_ssl_cert_file=NULL;
+char *cipher_list;
 
 #endif /* USE_SSL */
 
+        void
+doit(struct sockaddr_in *who);
+
+        int
 main(argc, argv)
+        int argc;
 	char *argv[];
 {
 	struct sockaddr_in from;
@@ -265,6 +282,10 @@ main(argc, argv)
 		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
 			    my_ssl_key_file=optarg+strlen("key=");
 			}
+
+		        if (strncmp(optarg,"cipher=",sizeof("cipher"))==0) {
+			    cipher_list=optarg+sizeof("cipher");
+			}
 			break;
 #endif /* USE_SSL */
 
@@ -452,7 +473,7 @@ main(argc, argv)
 
 #ifdef USE_SSL
         {
-	    int i;
+	  /*	    int i;*/
 
 	    /* if we are not running in debug then any error
 	     * stuff from SSL debug *must* not go down
@@ -469,10 +490,13 @@ main(argc, argv)
 		/* disable all the debug and trace */
 		SSL_LOG=SSL_ERR=NULL;
 	    }
+ 	    SSL_load_error_strings();
+	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
 
-            if (!X509_set_default_verify_paths()) {
+            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
 		fprintf(stderr,"telnetd: cannot load certificates via X509_set_default_verify_paths\n");
 		fflush(stderr);
+		ERR_print_errors(stderr);
 		exit(1);
 	    }
 
@@ -492,7 +516,7 @@ main(argc, argv)
 		usage();
 		/* NOT REACHED */
 	    } else if (argc == 1) {
-		    if (sp = getservbyname(*argv, "tcp")) {
+		    if ((sp = getservbyname(*argv, "tcp"))) {
 			sin.sin_port = sp->s_port;
 		    } else {
 			sin.sin_port = atoi(*argv);
@@ -634,44 +658,42 @@ main(argc, argv)
 
 #ifdef USE_SSL
         /* do the SSL stuff now ... before we play with pty's */
-	ssl_con=(SSL *)SSL_new();
+	ssl_con=(SSL *)SSL_new(ssl_ctx);
 	SSL_set_fd(ssl_con,0);
 
         /* just use telnetd for public and private and don't
 	 * encrypt it with a pass phrase 
 	 */
-	sprintf(ssl_file_path,"%s/private/%s",X509_get_default_cert_area(),
-	        "telnetd.key");
+	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
+	        "telnetd.pem");
 	if (SSL_use_RSAPrivateKey_file(ssl_con,
 	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file,
 		X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_RSAPrivateKey_file error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
-	    /* if there was a system error then display it */
-	    if (SSL_errno==SSL_R_ERRNO)
-		perror("SSL_use_RSAPrivateKey_file");
+	    fprintf(stderr,"SSL_use_RSAPrivateKey_file(%s) error ",
+	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file);
+	    ERR_print_errors(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
 	}
 
 	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.cert");
+	        "telnetd.pem");
 	if (SSL_use_certificate_file(ssl_con,
 	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file,
 	        X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_certificate_file error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
-	    /* if there was a system error then display it */
-	    if (SSL_errno==SSL_R_ERRNO)
-		perror("SSL_use_certificate_file");
+	    fprintf(stderr,"SSL_use_certificate_file(%s) error ",
+	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file);
+	    ERR_print_errors(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
 	}
 
+	/* if we gave a cipher list then ... */
+          if (cipher_list!=NULL)
+	    SSL_set_pref_cipher(ssl_con,cipher_list);
+
 	if (ssl_only_flag) {
 	    /* hmm ... only when running talking to things like
 	     * https servers should we hit this code and then
@@ -679,12 +701,11 @@ main(argc, argv)
 	     */
 	    SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
 
-	    if (!SSL_accept(ssl_con)) {
-		static char errbuf[1024];
-	    
-	        sprintf(errbuf,"SSL_accept error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
+	    if (SSL_accept(ssl_con) <= 0) {
+  		static char errbuf[1024];
+  	    
+	        sprintf(errbuf,"SSL_accept error %s\n",
+		    ERR_error_string(ERR_get_error()));
 
 		syslog(LOG_WARNING, errbuf);
 
@@ -704,6 +725,7 @@ main(argc, argv)
 	net = 0;
 	doit(&from);
 	/* NOTREACHED */
+	return 0;
 }  /* end of main */
 
 	void
@@ -770,6 +792,9 @@ static unsigned char ttytype_sbbuf[] = {
 	IAC, SB, TELOPT_TTYPE, TELQUAL_SEND, IAC, SE
 };
 
+    int
+terminaltypeok(char *s);
+
     int
 getterminaltype(name)
     char *name;
@@ -783,8 +808,20 @@ getterminaltype(name)
      * Handle the Authentication option before we do anything else.
      */
     send_do(TELOPT_AUTHENTICATION, 1);
-    while (his_will_wont_is_changing(TELOPT_AUTHENTICATION))
+    while (his_will_wont_is_changing(TELOPT_AUTHENTICATION)) {
+#ifdef TJH_FORCED_FLUSH
+        /* the following stuff *might* still be needed ...
+	 * but I don't recollect the exact problem that I 
+	 * was working around so I've ifdef-ed it out for
+	 * the moment --tjh
+	 */
+ 	netflush(); 
+#endif /* TJH_FORCED_FLUSH */
 	ttloop();
+#ifdef TJH_FORCED_FLUSH
+ 	netflush(); 
+#endif /* TJH_FORCED_FLUSH */
+    }
     if (his_state_is_will(TELOPT_AUTHENTICATION)) {
 	retval = auth_wait(name);
     }
@@ -1006,11 +1043,12 @@ extern void telnet P((int, int, char *));
 /*
  * Get a pty, scan input lines.
  */
+        void
 doit(who)
 	struct sockaddr_in *who;
 {
 	char *host, *inet_ntoa();
-	int t;
+	/*	int t;*/
 	struct hostent *hp;
 	int level;
 	int ptynum;
@@ -1071,14 +1109,18 @@ doit(who)
 	if (hp == NULL && registerd_host_only) {
 		fatal(net, "Couldn't resolve your address into a host name.\r\n\
          Please contact your net administrator");
+	} else if (hp) {
 #ifdef HAVE_UT_HOST
-	} else if (hp &&
-	    (strlen(hp->h_name) <= ((utmp_len < 0) ? -utmp_len : utmp_len))) {
-		host = hp->h_name;
+	  if (strlen(hp->h_name) <=
+	      ((utmp_len < 0) ? -utmp_len : utmp_len)) {
+	    hp->h_name[(utmp_len < 0) ? -utmp_len : utmp_len] = '\0';
+	  }
 #endif
+	  host = hp->h_name;
 	} else {
 		host = inet_ntoa(who->sin_addr);
 	}
+
 	/*
 	 * We must make a copy because Kerberos is probably going
 	 * to also do a gethost* and overwrite the static data...
@@ -1171,7 +1213,7 @@ telnet(f, p, host)
 	char *HE;
 	char *HN;
 	char *IM;
-	void netflush();
+	/*	void netflush();*/
 
 	/*
 	 * Initialize the slc mapping table.
