@@ -35,15 +35,18 @@
 static char sccsid[] = "@(#)utility.c	8.2 (Berkeley) 12/15/93";
 #endif /* not lint */
 
-/* 	$Id: utility.c,v 1.2 1995/07/27 12:58:32 martin Exp $	 */
+/* 	$Id: utility.c,v 1.3 1995/11/15 10:16:43 martin Exp $	 */
 
 #ifndef lint
-static char vcid[] = "$Id: utility.c,v 1.2 1995/07/27 12:58:32 martin Exp $";
+static char vcid[] = "$Id: utility.c,v 1.3 1995/11/15 10:16:43 martin Exp $";
 #endif /* lint */
 
 #define PRINTOPTIONS
 #include "telnetd.h"
 
+#include "../lib/libtelnet/auth.h"
+#include "../lib/libtelnet/auth-proto.h"
+
 /*
  * utility functions performing io related tasks
  */
@@ -60,7 +63,7 @@ static char vcid[] = "$Id: utility.c,v 1.2 1995/07/27 12:58:32 martin Exp $";
     void
 ttloop()
 {
-    void netflush();
+  /*    void netflush();*/
 
     DIAG(TD_REPORT, {sprintf(nfrontp, "td: ttloop\r\n");
 		     nfrontp += strlen(nfrontp);});
@@ -249,7 +252,7 @@ netclear()
  *		Send as much data as possible to the network,
  *	handling requests for urgent data.
  */
-    void
+    int
 netflush()
 {
     int n;
@@ -311,7 +314,7 @@ netflush()
     }
     if (n < 0) {
 	if (errno == EWOULDBLOCK || errno == EINTR)
-		return;
+		return 0;
 	cleanup(0);
     }
     nbackp += n;
@@ -328,7 +331,7 @@ netflush()
 	nclearto = 0;
 #endif	/* ENCRYPTION */
     }
-    return;
+    return 1;
 }  /* end of netflush */
 
 
